from invoke import Collection, Program
from invoke.config import Config, merge_dicts

from bigga import tasks

__VERSION__ = '0.2.0'


class TesterConfig(Config):

    prefix = 'rscli'

    @staticmethod
    def global_defaults():
        their_defaults = Config.global_defaults()
        my_defaults = {
            'run': {
                'echo': True,
            },
        }
        return merge_dicts(their_defaults, my_defaults)


program = Program(
    version=__VERSION__,
    config_class=TesterConfig,
    namespace=Collection.from_module(tasks)
)



from typing import Set, Dict
from json import dumps as jdumps, load as jload
from dataclasses import dataclass


class DataClassMixin(object):

    def dumps(self, **kwargs) -> str:
        return jdumps(self, default=lambda o: o.__dict__, **kwargs)

    @staticmethod
    def load(data: Dict):
        raise NotImplementedError('Please implement load()')


@dataclass
class Service(DataClassMixin):
    name: str
    framework: str
    args: Dict[str, str]

    @staticmethod
    def load(data: Dict):
        return Service(**data)


@dataclass
class Host(DataClassMixin):
    services: Set[str]
    service_args: Dict[str, str]

    ip: str = ''
    user: str = ''
    port: int = 0

    @staticmethod
    def load(data: Dict):
        return Host(**data)


@dataclass
class Bigga(DataClassMixin):
    name: str
    version: str
    language: str
    description: str
    hosts: Dict[str, Set[Host]]
    services: Set[Service]

    @staticmethod
    def load(data: Dict):
        hosts = {}
        for env, _hosts in data["hosts"].items():
            hosts[env] = [Host.load(host) for host in _hosts]
        data['hosts'] = hosts
        data['services'] = [
            Service.load(service) for service in data['services']]
        return Bigga(**data)


def get_bigga(bigga_file='.bigga.json'):
    data = jload(open(bigga_file))
    return Bigga.load(data)



def home_directory(cxn):
    """
    Get the absolute path to the user's home directory
    Example::
        from bigga.contrib.fabtools import home_directory
        home = home_directory(cxn, 'alice')
    """
    result = cxn.run('echo ~')
    return result.stdout.strip()



import json
import logging
import requests
from invoke import task
from posixpath import join
from fabric import Connection
from urllib.parse import quote

from patchwork import files as pw_files
from bigga.contrib.fabtools.user import home_directory
from bigga.contrib.patchwork.packages import update_index, packages

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)
BIGGA_DATA = json.load(open('.bigga.json'))
SSH_USERS = ['dhilipsiva']


def vagrant(c):
    config = {}
    output = c.run('vagrant ssh-config')
    __import__('ipdb').set_trace()
    for line in output.splitlines()[1:]:
        key, value = line.strip().split(' ', 1)
        config[key] = value
    print(config)
    # extra_args = _settings_dict(config)


def _settings_dict(config):
    settings = {}

    user = config['User']
    hostname = config['HostName']
    port = config['Port']

    # Build host string
    host_string = "%s@%s:%s" % (user, hostname, port)

    settings['user'] = user
    settings['hosts'] = [host_string]
    settings['host_string'] = host_string

    # Strip leading and trailing double quotes introduced by vagrant 1.1
    settings['key_filename'] = config['IdentityFile'].strip('"')

    settings['forward_agent'] = (config.get('ForwardAgent', 'no') == 'yes')
    settings['disable_known_hosts'] = True

    return settings


@task
def echo(c, env):  # , app, env):
    for host in BIGGA_DATA['hosts'][env]:
        cxn = Connection(host['ip'], user=host['user'])
        cxn.run('echo foo')


@task
def sync_auth_keys(c, env):
    # Add multiple public keys to the user's authorized SSH keys from GitHub.
    for host in BIGGA_DATA['hosts'][env]:
        cxn = Connection(host['ip'], user=host['user'])
        ssh_dir = join(home_directory(cxn), '.ssh')
        pw_files.directory(cxn, ssh_dir, mode='700')
        authorized_keys_filename = join(ssh_dir, 'authorized_keys')
        cxn.run('cat /dev/null > %s' % quote(authorized_keys_filename))
        logger.info('Fetching public keys from GitHub')
        for gh_user in SSH_USERS:
            r = requests.get('https://api.github.com/users/%s/keys' % gh_user)
            for key in r.json():
                cxn.run('echo %s >> %s' % (
                    quote(key['key']), quote(authorized_keys_filename)))


@task
def setup(c, env):
    for host in BIGGA_DATA['hosts'][env]:
        cxn = Connection(host['ip'], user=host['user'])
        update_index(cxn)
        packages(cxn, ["neovim", "python-pip"])



from bigga.contrib.patchwork.info import distro_family


def update_index(cxn):
    """
    Update APT package definitions.
    """
    if distro_family(cxn) == "debian":
        cxn.run('DEBIAN_FRONTEND=noninteractive sudo apt-get update')
    else:
        cxn.run('sudo yum update')


def is_installed(cxn, pkg_name):
    """
    Check if a package is installed.
    """
    # TODO: support yum
    res = cxn.run("dpkg -s %(pkg_name)s" % locals())
    res = res.result.strip()
    for line in res.splitlines():
        if line.startswith("Status: "):
            status = line[8:]
            if "installed" in status.split(' '):
                return True
    return False


def package(c, *packages):
    """
    Installs one or more ``packages`` using the system package manager.
    Specifically, this function calls a package manager like ``apt-get`` or
    ``yum`` once per package given.
    """
    # Try to suppress interactive prompts, assume 'yes' to all questions
    apt = "DEBIAN_FRONTEND=noninteractive apt-get install -y {}"
    # Run from cache vs updating package lists every time; assume 'yes'.
    yum = "yum install -y %s"
    manager = apt if distro_family(c) == "debian" else yum
    for package in packages:
        c.sudo(manager.format(package))


def packages(cxn, package_list):
    for _package in package_list:
        package(cxn, _package)



from setuptools import setup, find_packages

__VERSION__ = '0.2.0'

setup(
    name='bigga',
    version=__VERSION__,
    description="A simple CLI tool to deploy Python & Node microservices on linux instaces. Built on top of invoke, fabric and patchwork.",  # NOQA
    long_description="Bigga - A simple, opinionanted deployment tool for Python & Node",  # NOQA
    url='https://github.com/reckonsys/bigga',
    author='dhilipsiva',
    author_email='dhilipsiva@pm.me',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
    ],

    keywords='bigga reckonsys cli deploy deployment',
    packages=find_packages(),
    install_requires=[
        'invoke', 'fabric', 'patchwork'
    ],
    entry_points={
        'console_scripts': ['bigga = bigga.cli:program.run']
    }
)
